DECLARE_DFHANDLE(DFSFILEWRAP);




/****************************************************************************/
/****************************************************************************/

/*
szMagicText : Texte de prsentation du format DFS (avec URL). DfCrcMagicText est le CRC32 de ce texte. (on s'inspirera du debut du PNG pour protger contre les accidents de conversion bin/ascii)
dfMagic : un code Magic 'GVDF'.
dfVersionForCreate : permet de connatre la version utilise pour crer le fichier.
dfVersionNeeded : version ncessaire  l'ouverture du fichier.
dfsTotalSize : taille du fichier DFS hors en-tte. (o 0 pour DFSINFOBEGIN dans le cas stream)
dfsThisFileSize : taille du fichier volume DFS courant (o 0 pour DFSINFOBEGIN dans le cas stream). Diffrent de dfsTotalSize uniquement pour le multivolume.
dfsThisVolumeNumber : numro du fichier courant dans l'ordre des volumes. (0 pour le cas stream, 1 pour le cas illimit)
dfsTotalNumberOfVolume : nombre total de volume (0 pour le cas stream, 1 pour le cas taille illimit)
dfsVolumeSize : taille d'un volume (0 pour le cas stream ou illimit).
*/


typedef struct
{
  dfuLong32 sizeStruct;
  dfuLong32 dfStatus;
  const void *filename;         /* filename or other info for open callback */
}
DFSFILEINFOPARAMINTERNAL;

DFSFILEWRAP DfsOpenFile(DFSFILEINFOPARAMINTERNAL DfsFileParam, H_ERROR_INFO* pei);
/*
Ouvre un fichier DFS.
Paramtres :
dfsStatus : Voir DfsGetStatus
Valeur de retour : handle du Fichier DFS
*/

dfuLong32 DfsFlushWriteFile(DFSFILEWRAP dfsFileWrap,H_ERROR_INFO* pei);
/*
Flush les donne du fichier DFS.
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Valeur de retour : 0
*/


dfuLong32 DfsCloseFile(DFSFILEWRAP dfsFileWrap,H_ERROR_INFO* pei);
/*
Ferme un fichier DFS.
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Valeur de retour : 0
*/

dfuLong32 DfsGetStatus(DFSFILEWRAP dfsFileWrap,H_ERROR_INFO* pei);
/*
Retourne les flag permettant de savoir ce qui est possible avec le fichier.
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Valeur de retour : Combinaison des valeurs suivantes
DFS_STREAM :    implique qu'un seek est impossible.
DFS_READABLE :  Fichier permettant de lire
DFS_WRITABLE :  Fichier permettant d'crire
DFS_NEWFILE :   Nouveau fichier. Ne peut tre prsent que si DFS_WRITABLE est prsent
*/
// Un fichier DFS_STREAM est soit DFS_READABLE soit DFS_WRITABLE | DFS_NEWFILE



void DfsGetSize(DFSFILEWRAP dfsFileWrap,dfuLong32 *posLow, dfuLong32 *posHigh,H_ERROR_INFO* pei);
/*
Retourne la taille d'un fichier DFS.
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Valeur de retour : Taille du fichier DFS en octets

Un fichier avec le flag DFS_STREAM retournera DFS_ERROR_BADPARAMETER
*/

dfuLong32 DfsSeek(DFSFILEWRAP dfsFileWrap, dfuLong32 posLow, dfuLong32 posHigh,H_ERROR_INFO* pei);
dfuLong32 DfsSeekMulAlign(DFSFILEWRAP dfsFileWrap, dfuLong32 posLow, dfuLong32 posHigh,H_ERROR_INFO* pei);
/*
Positionne le pointeur courant dans le fichier DFS ( partir du dbut du fichier).
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Valeur de retour : 0 en cas de succs (DFS_SUCCESS)

Un fichier avec le flag DFS_STREAM retournera DFS_ERROR_BADPARAMETER
*/

dfuLong32 DfsRead(DFSFILEWRAP dfsFileWrap, void *Buf,
                  dfuLong32 Size, dfuLong32 * errorCode,H_ERROR_INFO* pei);
/*
Effectue une lecture dans le fichier .DFS
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Buf :   Pointeur vers le buffer recevant les donnes
Size :  Nombre d'octet  lire
errorCode : Pointeur vers une variable recevant le code d'erreur (peut tre NULL)
Valeur de retour : Nombre d'octets lus.

Un fichier sans le flag DFS_READABLE retournera 0 et *errorCode  DFS_ERROR_BADPARAMETER
Si le fichier est termin, *errorcode est  DFS_ERROR_EOF, sinon 0
Avec Size = 0, on peut utiliser cette fonction pour savoir si la fin du fichier est atteinte.
*/

dfuLong32 DfsWrite(DFSFILEWRAP dfsFileWrap, const void *Buf,
                   dfuLong32 Size, dfuLong32 * errorCode,H_ERROR_INFO* pei);
/*
Effectue une criture dans le fichier .DFS
Paramtres :
DfsFileWrap :   Handle du fichier DFS
Buf :   Pointeur vers le buffer contenant les donnes
Size :  Nombre d'octet  lire
errorCode : Pointeur vers une variable recevant le code d'erreur (peut tre NULL)
Valeur de retour : Nombre d'octets crits.

Un fichier sans le flag DFS_WRITABLE retournera 0 et *errorCode  DFS_ERROR_BADPARAMETER
*/

void DfsTell(DFSFILEWRAP dfsFileWrap, dfuLong32 *posLow, dfuLong32* posHigh, dfuLong32 * errorCode,H_ERROR_INFO* pei);
/* Donne la position courante dans le fichier */

void DfsTellDivAlign(DFSFILEWRAP dfsFileWrap, dfuLong32 *posLow, dfuLong32* posHigh, dfuLong32 * errorCode,H_ERROR_INFO* pei);
/* Donne la position courante dans le fichier */

void DfsMarkEndNow(DFSFILEWRAP dfsFileWrap, dfuLong32 * errorCode,H_ERROR_INFO* pei);
/* Marque la position actuelle comme fin du fichier */

dfuLong32 DfsReadOrWriteFillAlign(DFSFILEWRAP dfsFileWrap, BOOL fWrite,
                                dfuLong32 * errorCode,H_ERROR_INFO* pei);
/* Aligne la position courante dans le fichier (en crivant des zro si neccessaire) */
